# -*- coding: utf-8 -*-

import logging
import json
import os
import signal
from .gproto import send_communication_msg, file_upload
from . import config
from .utils import Counter
from _queue import Empty
from .exceptions import FilePathError

logger = logging.getLogger("sdk.gomer.base")


class Base(object):

    @staticmethod
    def _edit_msg(seq=None, msgtype=1, code=None, num=None, item=None, prm1=None, prm2=None, prm3=None, prm4=None,
                 prm5=None, prmstr1=None, prmstr2=None, prmstr3=None, prmstr4=None, prmstr5=None):
        msg = {"seq": seq, "msgtype": msgtype, "code": code, "sdks":
               {"num": num, "item": item, "prm1": prm1, "prm2": prm2, "prm3": prm3, "prm4": prm4,"prm5": prm5,
                "prmstr1": prmstr1, "prmstr2": prmstr2, "prmstr3": prmstr3, "prmstr4": prmstr4, "prmstr5": prmstr5}
               }
        return msg

    @staticmethod
    def _edit_transmit_msg(seq, type, name, size):
        msg = {
            "seq": seq, "msgtype": 1,
            "transmit": {"type": type, "status": 100, "name": name, "size": size}
        }
        return msg

    def _get_dict_value(self, dict1, obj, default=None):
        for key, value in dict1.items():
            if key == obj:
                return value
            else:
                if type(value) is dict:
                    result = self._get_dict_value(value, obj)
                    if result is not default:
                        return result
        return default

    @staticmethod
    def _message_transmit(msg, subscriber):
        msg = msg
        subscriber = subscriber
        subscriber.subscribe()
        subscriber.send_msg(msg)
        return subscriber.get_result()

    @staticmethod
    def _kill_process():
        pid = os.getpid()
        os.kill(pid, signal.SIGTERM)

    @staticmethod
    def _isfile(path):
        return os.path.isfile(path)

    @staticmethod
    def _get_file_size(path):
        return os.path.getsize(path)

    @staticmethod
    def _get_file_name(path):
        return os.path.basename(path)

    @staticmethod
    def _path_join(length, width, name):
        return ''.join(("sdk1_",str(length), "x", str(width),"_", name))







