# -*- coding: utf-8 -*-


# -*- coding: utf-8 -*-

import time
import logging
import random
from threading import Lock
from sdk.gomer.gproto import *
from sdk.gomer import config
from .utils import get_dict_value, Counter
from .exceptions import NoDevicesFound, ChangeModeError
from .handler import message_center
from _queue import Empty


logger = logging.getLogger("sdk.gomer.conn")


class DeviceConnector(object):
    """Connection device class"""

    _instance_lock = Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(DeviceConnector, "_instance"):
            with DeviceConnector._instance_lock:
                if not hasattr(DeviceConnector, "_instance"):
                    DeviceConnector._instance = object.__new__(cls)
        return DeviceConnector._instance

    def __init__(self):
        self.search_socket_id = search_socket_create()
        self.device_list = []
        self.sid = random.randint(100000, 999999)

    def connect_device(self, device_name):
        self._recv_search_socket_msg()
        start_time = time.time()
        while True:
            search_device(self.search_socket_id)
            time.sleep(5)
            if device_name in self.device_list:
                self._select_device(device_name)
                self._setup_handshake(device_name)
                return communication_socket_create(), self.sid
            elif time.time() - start_time > 15:
                raise NoDevicesFound("The device was not found")

    def _recv_search_socket_msg(self):
        """Receive search_socket message"""
        s = SearchMsgRecvThread(self.search_socket_id, self.device_list)
        s.start()

    def _select_device(self, device_name):
        select_device(device_name, self.sid)

    def _setup_handshake(self, device_name):
        seq = Counter().get_seq()
        msg = {"seq": seq, "msgtype": 1, "hard": {"call": 102, "sid": self.sid, "did": device_name}}
        send_search_msg(self.search_socket_id, msg)
        while True:
            if config.HANDSHAKE_FLAG:
                search_socket_destroy(self.search_socket_id)
                return
            time.sleep(0.2)


class DeviceCommunication(object):

    _instance_lock = Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(DeviceCommunication, "_instance"):
            with DeviceCommunication._instance_lock:
                if not hasattr(DeviceCommunication, "_instance"):
                    DeviceCommunication._instance = object.__new__(cls)
        return DeviceCommunication._instance

    def __init__(self, communication_socket_id, sid):
        self.communication_socket_id = communication_socket_id
        self.sid = sid

    def start(self):
        msg_recv_thread = CommunicationMsgRecvThread(self.communication_socket_id)
        msg_recv_thread.start()
        heart_recv_thread = HeartBeatRecvThread(self.communication_socket_id)
        heart_recv_thread.start()
        heart_send_thread = HeartBeatSendThread(self.communication_socket_id, self.sid)
        heart_send_thread.start()
        self._change_to_sdk_mode()
        message_center.start_message_process(config.sdk_queue)

    def _change_to_sdk_mode(self):
        """enter into sdk mode request"""
        seq = Counter().get_seq()
        msg = {"seq": seq, "msgtype": 1, "control": {"mode": 106}}
        send_communication_msg(msg, self.communication_socket_id)
        while True:
            try:
                value = config.control_queue.get(timeout=30)
            except Empty:
                raise ChangeModeError("change to sdk mode failed")
            if value.get("code") is 100:
                if get_dict_value(value, "mode") is 106:
                    logger.info("enter into sdk mode succeed")
                    time.sleep(2)
                    return























