# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import message_center, Subscriber

logger = logging.getLogger("sdk.gomer.emotion")


class Emotion(Base):
    """Emotion-related interface, providing a variety of emotional display interface,
       including user-defined emotional editing."""

    __slots__ = ("_name", "_expression_switch", "_voice_switch", "_action_switch", "_paw_safe")

    bool_dict = {
        True: 101,
        False: 100
    }

    def __init__(self):
        self._name = None
        self._expression_switch = True
        self._voice_switch = True
        self._action_switch = True
        self._paw_safe = True

    @property
    def paw_safe(self):
        return self._paw_safe

    @paw_safe.setter
    def paw_safe(self, value):
        if not value:
            value = self.paw_safe
        if value not in (True, False):
            logger.error("Parameter paw_safe type error")
            raise ParameterError("Parameter paw_safe type error")
        self._paw_safe = value

    @property
    def action_switch(self):
        return self._action_switch

    @action_switch.setter
    def action_switch(self, value):
        if not value:
            value = self.action_switch
        if value not in (True, False):
            logger.error("Parameter action_switch type error")
            raise ParameterError("Parameter action_switch type error")
        self._action_switch = value

    @property
    def voice_switch(self):
        return self._voice_switch

    @voice_switch.setter
    def voice_switch(self, value):
        if not value:
            value = self.voice_switch
        if value not in (True, False):
            logger.error("Parameter voice_switch type error")
            raise ParameterError("Parameter voice_switch type error")
        self._voice_switch = value

    @property
    def expression_switch(self):
        return self._expression_switch

    @expression_switch.setter
    def expression_switch(self, value):
        if not value:
            value = self.expression_switch
        if value not in (True, False):
            logger.error("Parameter expression_switch type error")
            raise ParameterError("Parameter expression_switch type error")
        self._expression_switch = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    def emotion_show(self):
        item = 1401
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item,  prm1=self.bool_dict.get(self.expression_switch),
                                                    prm2=self.bool_dict.get(self.voice_switch),
                                                    prm3=self.bool_dict.get(self.action_switch),
                                                    prm4=self.bool_dict.get(self.paw_safe), prmstr1=self.name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="emotion_show", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def emotion_status(self):
        item = 1402
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="emotion_status", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def stop_emotion(self):
        item = 1404
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="stop_emotion", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result


