# -*- coding: utf-8 -*-

from . import config
from queue import Queue, Empty
from threading import *
from .utils import get_dict_value


class EventManager(object):
    """Event manager class"""
    def __init__(self):
        self.__eventQueue = Queue()
        self.__active = False
        self.__thread = Thread(target=self.__run)
        self.__handlers = {}

    def __run(self):
        """The engine running"""
        while self.__active:
            try:
                event = self.__eventQueue.get(block=True, timeout=1)
                self.__event_process(event)
            except Empty:
                pass

    def __event_process(self, event):
        """handle event"""
        if event.type_ in self.__handlers:
            for handler in self.__handlers[event.type_]:
                print("8888888888888888888888888888888888888888888888888")
                handler()

    def start(self):
        """start"""
        self.__active = True
        m = Thread(target=self.msg_listener)
        m.start()
        self.__thread.start()

    def stop(self):
        """stop"""
        self.__active = False
        self.__thread.join()

    def add_event_listener(self, type_, handler):
        """Bind event and listener handler functions"""
        try:
            handler_list = self.__handlers[type_]
        except KeyError:
            handler_list = []
        self.__handlers[type_] = handler_list
        if handler not in handler_list:
            handler_list.append(handler)

    def remove_event_listener(self, type_, handler):
        """Removes the listener's handler function"""
        try:
            handler_list = self.__handlers[type_]
            if handler in handler_list:
                handler_list.remove(handler)
        except KeyError:
            handler_list = []

    def query_event_listener(self):
        return 0, self.__handlers

    def send_event(self, event):
        """Sends an event and stores it to the event queue"""
        self.__eventQueue.put(event)

    def get_queue(self):
        print(self.__eventQueue.qsize(),self.__eventQueue)

    def msg_listener(self):
        """Listen to the message"""
        while True:
            event_msg = config.event_queue.get()
            print(event_msg,'=======================================================')
            try:
                if 2001 == get_dict_value(event_msg, "item"):
                    if 101 == get_dict_value(event_msg, "prm1"):  # The button was pressed once
                        event_o = Event("button_click")
                        event_o.dict["button"] = "click"
                        self.send_event(event_o)
                    elif 102 == get_dict_value(event_msg, "prm1"):  # The button was pressed twice
                        event_o = Event("button_click_twice")
                        event_o.dict["button"] = "double_click"
                        self.send_event(event_o)
                elif 2002 == get_dict_value(event_msg, "item"):
                    if 10 == get_dict_value(event_msg, "prm1"):  # power is very low
                        event_o = Event("very_low_power")
                        event_o.dict["power"] = "very_low_power"
                        self.send_event(event_o)
                    elif 20 == get_dict_value(event_msg, "prm1"):  # low power
                        event_o = Event("low_power")
                        event_o.dict["power"] = "low_power"
                        self.send_event(event_o)
                    elif 100 == get_dict_value(event_msg, "prm2"):  # normal power
                        event_o = Event("normal_power")
                        event_o.dict["power"] = "normal_power"
                        self.send_event(event_o)
                elif 2003 == get_dict_value(event_msg, "item"):
                    if 101 == get_dict_value(event_msg, "prm1"):  # paw issue
                        event_o = Event("paw_issue")
                        event_o.dict["movement"] = "paw_issue"
                        self.send_event(event_o)
                    elif 102 == get_dict_value(event_msg, "prm1"):  # forearm issue
                        event_o = Event("forearm_issue")
                        event_o.dict["movement"] = "forearm_issue"
                        self.send_event(event_o)
                    elif 103 == get_dict_value(event_msg, "prm1"):  # arm issue
                        event_o = Event("arm_issue")
                        event_o.dict["movement"] = "arm_issue"
                        self.send_event(event_o)
                    elif 104 == get_dict_value(event_msg, "prm1"):  # head issue
                        event_o = Event("head_issue")
                        event_o.dict["movement"] = "head_issue"
                        self.send_event(event_o)
                    elif 108 == get_dict_value(event_msg, "prm1"):  # foot issue
                        event_o = Event("foot_issue")
                        event_o.dict["movement"] = "foot_issue"
                        self.send_event(event_o)
                elif 2004 == get_dict_value(event_msg, "itme"):  # posture change
                    event_o = Event("posture_change")
                    event_o.dict["posture"] = "posture_change"
                    self.send_event(event_o)
                else:
                    continue
            except Exception:
                continue


class Event(object):
    """The event message is automatically fed back by the robot, and the user processes the event by himself."""
    def __init__(self, type_=None):
        self.eventManager = None
        self.type_ = type_
        self.dict = {}

    def open_manager(self):
        """Start the event manager thread"""
        self.eventManager = EventManager()
        self.eventManager.start()
        return 0, None

    def close_manager(self):
        """close the event manager thread"""
        self.eventManager.stop()
        return 0, None

    def bind_event_callback(self, event_type, callback_func):
        """bind event

        Args:
            event_type: Contains "button_click","button_click_twice","very_low_power","low_power","normal_power",
                                    "paw_issue","forearm_issue","arm_issue","head_issue,foot_issue,posture_change
            callback_func: Event callback function.
        """
        self.eventManager.add_event_listener(event_type, callback_func)
        return 0, None

    def unbind_event_callback(self, event_type, callback_func):
        """unbind event

        Args:
            event_type: Contains "button_click","button_click_twice","very_low_power","low_power","normal_power",
                                    "paw_issue","forearm_issue","arm_issue","head_issue,foot_issue,posture_change
            callback_func: Event callback function.
        """
        self.eventManager.remove_event_listener(event_type, callback_func)
        return 0, None

    def query_for_bound_events(self):
        """query event

        Returns:
            (status, value): status is the status code, success is 0, failure is -1; value is the return value,
                            which a dict {"event_type": [callback_func1, callback_func2, ...], ...}
                """
        return self.eventManager.query_event_listener()


"""
事件模块用法介绍:
1,event.open_manager()
    open_manager函数用于创建一个事件管理器对象,并开启一个线程运行事件管理器
2,event.close_manager()
    close_manager函数由于关闭事件管理器
3,event.bind_event_callback(event_type, callback_func)
    bind_event_callback函数用于绑定事件,只接收两个参数,event_type为需要绑定的事件类型,如:"button_click",
    callback_func为自定义的一个函数,此函数无参数,当sdk接收到gomer端触发的事件时,将执行callback_func函数;
    同种类型的事件可以绑定多个回个回调函数,当sdk接收到gomer端触发的事件时,将按绑定的先后顺序依次执行callback_func函数
4,event.unbind_event_callback(event_type, callback_func)
    unbind_vent_callback函数用于解绑事件,只接收两个函数,event_type为已绑定的事件类型,callback_func为已绑定在该事件类型
    中的回调函数,解绑后,当sdk接收到gomer端触发的该类型事件时,将不再执行已解绑的回调函数
"""
